package com.gree.mall.miniapp.logic.material.parts;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import com.gree.mall.miniapp.bean.material.WebsitPartsSalesBean;
import com.gree.mall.miniapp.bean.material.WebsitPartsSalesItemBean;
import com.gree.mall.miniapp.bean.material.stock.WebsitStockDTO;
import com.gree.mall.miniapp.bean.material.stock.WorkerStockDTO;
import com.gree.mall.miniapp.enums.material.*;
import com.gree.mall.miniapp.logic.material.stock.MaterialGoodsStockLogic;
import com.gree.mall.miniapp.plus.entity.WebsitPartsSales;
import com.gree.mall.miniapp.plus.entity.WebsitPartsSalesItem;
import com.gree.mall.miniapp.plus.entity.WebsitSalesPayOrder;
import com.gree.mall.miniapp.plus.service.WebsitPartsSalesItemService;
import com.gree.mall.miniapp.plus.service.WebsitPartsSalesService;
import io.jsonwebtoken.lang.Collections;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
public class WebsitPartsSalesOrderLogic {

    @Resource
    WebsitPartsSalesService websitPartsSalesService;
    @Resource
    WebsitPartsSalesItemService websitPartsSalesItemService;
    @Resource
    MaterialGoodsStockLogic materialGoodsStockLogic;

    public void handlePaySuccessMethod(WebsitSalesPayOrder order) throws Exception {
        // 更新配件销售单状态为已支付
        WebsitPartsSales partsSalesOrder = websitPartsSalesService.lambdaQuery()
                .eq(WebsitPartsSales::getId, order.getOrderId())
                .one();

        log.info("配件销售单状态：" + partsSalesOrder.getStatus());
        if (PartsOrderStatusEnum.SUBMIT.getKey().equals(partsSalesOrder.getStatus())) {
            // 销售单状态是提交状态执行
            partsSalesOrder.setPayType(order.getPayType())
                    .setPayFlag(PartsPayFlagEnum.PAY_NOT_TAKE.getKey())
                    .setStatus(PartsOrderStatusEnum.PAYED.getKey())
                    .setPayTime(order.getPayTime())
                    .setTransactionId(order.getTransactionId())
                    .setOpenid(order.getOpenid());


            List<WebsitPartsSalesItem> partsSalesOrderItems = websitPartsSalesItemService.lambdaQuery()
                    .eq(WebsitPartsSalesItem::getSalesId, order.getOrderId())
                    .list();

            if (!partsSalesOrder.getIsApply()) {
                WebsitPartsSalesBean partsSalesOrderBean = new WebsitPartsSalesBean();
                BeanUtils.copyProperties(partsSalesOrder, partsSalesOrderBean);
                List<WebsitPartsSalesItemBean> partsSalesOrderItemBeanList = BeanUtil.copyToList(partsSalesOrderItems, WebsitPartsSalesItemBean.class);

                partsSalesOrderBean.setItemList(partsSalesOrderItemBeanList);

                this.handleForwardShopAndWorkerStock(partsSalesOrderBean);
            }

            partsSalesOrder.updateById();
        }
    }

    private void handleForwardShopAndWorkerStock(WebsitPartsSalesBean partsSalesOrderBean) throws Exception {
        // 扣减网点库存
        List<WebsitStockDTO> websitStockDTOList = this.getWebsitStock(partsSalesOrderBean, DirectFlagEnum.SUB);
        materialGoodsStockLogic.handleWebsitStock(websitStockDTOList);

        if (DeliveryTypeEnum.EXPRESS.getValue().equals(partsSalesOrderBean.getDeliveryType())
                && BuyPeopleEnum.WORKER.getKey().equals(partsSalesOrderBean.getBuyPeople())) {
            // 如果是快递并且销售类型为“服务人员”的单，需要增加师傅在途库存
            // 增加师傅在途库存
            List<WorkerStockDTO> workerStockDTOList = this.getWorkerStock(partsSalesOrderBean,
                    PartsRefTypeEnum.SALES.toString(), null, PartsAttrEnum.WAY.toString());

            materialGoodsStockLogic.handleWorkerStock(workerStockDTOList);
        }
    }

    public List<WebsitStockDTO> getWebsitStock(WebsitPartsSalesBean partsSalesOrderBean, DirectFlagEnum directFlag){
        if(Collections.isEmpty(partsSalesOrderBean.getItemList())) {
            return null;
        }
        List<WebsitStockDTO> websitStockDTOList = new ArrayList<>();
        for(WebsitPartsSalesItem item : partsSalesOrderBean.getItemList()) {
            WebsitStockDTO websitStockDTO = new WebsitStockDTO();
            websitStockDTO.setCompanyWechatId(partsSalesOrderBean.getCompanyWechatId());
            websitStockDTO.setCompanyWechatName(partsSalesOrderBean.getCompanyWechatName());
            websitStockDTO.setWebsitId(partsSalesOrderBean.getWebsitId());
            websitStockDTO.setWebsitName(partsSalesOrderBean.getWebsitName());
            websitStockDTO.setSdate(new Date());
            websitStockDTO.setPartsWebsitId(item.getPartsWebsitId());
            websitStockDTO.setGoodsId(item.getPartsNumber());
            websitStockDTO.setGoodsName(item.getPartsName());
            websitStockDTO.setMaterialGroupName(item.getMaterialGroupName());
            websitStockDTO.setPrice(item.getMarketPrice());
            websitStockDTO.setGoodsStockUnit(item.getGoodsStockUnit());
            websitStockDTO.setVoucherNo(partsSalesOrderBean.getId());
            websitStockDTO.setApplyNo(partsSalesOrderBean.getId());
            websitStockDTO.setObj(PartsRefTypeEnum.SALES.getKey());
            websitStockDTO.setPartsAttr(PartsAttrEnum.NEW.getKey());
            websitStockDTO.setChangeQty(item.getQty());
            websitStockDTO.setDirectFlag(directFlag.getKey());
            websitStockDTO.setRemark(partsSalesOrderBean.getRemark());
            websitStockDTO.setOperateBy(partsSalesOrderBean.getSubmitBy());
            websitStockDTO.setOperateTime(new Date());
            websitStockDTO.setStorageId(partsSalesOrderBean.getStorageId());
            websitStockDTOList.add(websitStockDTO);
        }
        return websitStockDTOList;
    }

    public List<WorkerStockDTO> getWorkerStock(WebsitPartsSalesBean partsSalesOrderBean, String refType, String partsAttr1,
                                               String partsAttr2){
        if(Collections.isEmpty(partsSalesOrderBean.getItemList())) {
            return null;
        }
        List<WorkerStockDTO> workerStockDTOList = new ArrayList<>();

        for (WebsitPartsSalesItem item : partsSalesOrderBean.getItemList()) {
            if (StringUtils.isNotBlank(partsAttr1)) {
                WorkerStockDTO workerStockDTO = new WorkerStockDTO();
                workerStockDTO.setCompanyWechatId(partsSalesOrderBean.getCompanyWechatId());
                workerStockDTO.setCompanyWechatName(partsSalesOrderBean.getCompanyWechatName());
                workerStockDTO.setIdentity(item.getIdentity());
                workerStockDTO.setWorkerId(partsSalesOrderBean.getWorkerId());
                workerStockDTO.setWorkerName(partsSalesOrderBean.getWorkerName());
                workerStockDTO.setWebsitId(partsSalesOrderBean.getWebsitId());
                workerStockDTO.setWebsitName(partsSalesOrderBean.getWebsitName());
                workerStockDTO.setSdate(DateUtil.date());
                workerStockDTO.setPartsWebsitId(item.getPartsWebsitId());
                workerStockDTO.setGoodsId(item.getPartsNumber());
                workerStockDTO.setGoodsName(item.getPartsName());
                workerStockDTO.setMaterialGroupName(item.getMaterialGroupName());
                workerStockDTO.setPrice(item.getMarketPrice());
                workerStockDTO.setGoodsStockUnit(item.getGoodsStockUnit());
                workerStockDTO.setRef(partsSalesOrderBean.getId());
                workerStockDTO.setRefType(refType);
                workerStockDTO.setPartsAttr(partsAttr1);
                workerStockDTO.setChangeQty(item.getQty());
                workerStockDTO.setDirectFlag(DirectFlagEnum.SUB.getKey());
                workerStockDTO.setRemark(partsSalesOrderBean.getRemark());
                workerStockDTO.setOperateBy(partsSalesOrderBean.getWorkerName());
                workerStockDTO.setOperateTime(DateUtil.date());
                workerStockDTOList.add(workerStockDTO);
            }
            if (StringUtils.isNotBlank(partsAttr2)) {
                WorkerStockDTO workerStockDTO = new WorkerStockDTO();
                workerStockDTO.setIdentity(item.getIdentity());
                workerStockDTO.setWorkerId(partsSalesOrderBean.getWorkerId());
                workerStockDTO.setWorkerName(partsSalesOrderBean.getWorkerName());
                workerStockDTO.setWebsitId(partsSalesOrderBean.getWebsitId());
                workerStockDTO.setWebsitName(partsSalesOrderBean.getWebsitName());
                workerStockDTO.setSdate(DateUtil.date());
                workerStockDTO.setPartsWebsitId(item.getPartsWebsitId());
                workerStockDTO.setGoodsId(item.getPartsNumber());
                workerStockDTO.setGoodsName(item.getPartsName());
                workerStockDTO.setMaterialGroupName(item.getMaterialGroupName());
                workerStockDTO.setPrice(item.getMarketPrice());
                workerStockDTO.setGoodsStockUnit(item.getGoodsStockUnit());
                workerStockDTO.setRef(partsSalesOrderBean.getId());
                workerStockDTO.setRefType(refType);
                workerStockDTO.setPartsAttr(partsAttr2);
                workerStockDTO.setChangeQty(item.getQty());
                workerStockDTO.setDirectFlag(DirectFlagEnum.ADD.getKey());
                workerStockDTO.setRemark(partsSalesOrderBean.getRemark());
                workerStockDTO.setOperateBy(partsSalesOrderBean.getWorkerName());
                workerStockDTO.setOperateTime(DateUtil.date());
                workerStockDTOList.add(workerStockDTO);
            }
        }
        return workerStockDTOList;
    }
}
